'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getChangedDeselection = exports.getChangedSelection = exports.getSelection = exports.getItems = exports.rangeTo = exports.removeAll = exports.remove = exports.toggle = exports.replace = exports.setSelection = exports.setItems = exports.init = undefined;

var _fp = require('lodash/fp');

function init() {
  return {
    items: [],
    selected: [],
    changed: {
      selected: [],
      deselected: []
    },
    anchor: null
  };
}

var setItems = (0, _fp.curry)(function (itemsIterable, state) {
  var items = _iterableToArray(itemsIterable);
  return {
    items: items,
    selected: (0, _fp.intersection)(state.selected, items),
    changed: {
      selected: [],
      deselected: (0, _fp.difference)(state.selected, items)
    },
    anchor: (0, _fp.includes)(state.anchor, items) ? state.anchor : null
  };
});

var setSelection = (0, _fp.curry)(function (selectedItems, state) {
  var selection = (0, _fp.intersection)(selectedItems, state.items);
  return {
    items: state.items,
    selected: selection,
    changed: {
      selected: (0, _fp.without)(state.selected, selection),
      deselected: (0, _fp.without)(selection, state.selected)
    },
    anchor: state.anchor
  };
});

function getItems(state) {
  return (0, _fp.clone)(state.items);
}

function getSelection(state) {
  return (0, _fp.clone)(state.selected);
}

function getChangedSelection(state) {
  return (0, _fp.clone)(state.changed.selected);
}

function getChangedDeselection(state) {
  return (0, _fp.clone)(state.changed.deselected);
}

function _getAnchor(state) {
  if (state.anchor !== null && state.anchor !== undefined) {
    return state.anchor;
  }

  if (state.selected.length > 0) {
    return _getBottommostSelectedItem(state);
  }

  return state.items[0];
}

function _getBottommostSelectedItem(state) {
  var previousItem = null;

  var isSelected = function isSelected(item) {
    return state.selected.indexOf(item) !== -1;
  };

  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    for (var _iterator = state.items[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      var item = _step.value;

      if (isSelected(item)) {
        previousItem = item;
      }
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator.return) {
        _iterator.return();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }

  return previousItem;
}

function _iterableToArray(iterable) {
  var array = [];
  var _iteratorNormalCompletion2 = true;
  var _didIteratorError2 = false;
  var _iteratorError2 = undefined;

  try {
    for (var _iterator2 = iterable[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
      var item = _step2.value;

      array.push(item);
    }
  } catch (err) {
    _didIteratorError2 = true;
    _iteratorError2 = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion2 && _iterator2.return) {
        _iterator2.return();
      }
    } finally {
      if (_didIteratorError2) {
        throw _iteratorError2;
      }
    }
  }

  return array;
}

var replace = (0, _fp.curry)(function (selectedItem, state) {
  if (!(0, _fp.includes)(selectedItem, state.items)) {
    return state;
  }
  return {
    items: state.items,
    selected: [selectedItem],
    changed: {
      selected: (0, _fp.without)(state.selected, [selectedItem]),
      deselected: (0, _fp.without)([selectedItem], state.selected)
    },
    anchor: selectedItem
  };
});

var toggle = (0, _fp.curry)(function (toggledItem, state) {
  if (!(0, _fp.includes)(toggledItem, state.items)) {
    return state;
  }

  var itemIsAdded = !(0, _fp.includes)(toggledItem, state.selected);

  if (itemIsAdded) {
    return {
      items: state.items,
      selected: state.selected.concat([toggledItem]),
      changed: {
        selected: [toggledItem],
        deselected: []
      },
      anchor: toggledItem
    };
  }

  var anchorIsRemoved = toggledItem === state.anchor;
  var newAnchor = anchorIsRemoved ? null : state.anchor;

  return {
    items: state.items,
    selected: (0, _fp.without)([toggledItem], state.selected),
    changed: {
      selected: [],
      deselected: [toggledItem]
    },
    anchor: newAnchor
  };
});

var remove = (0, _fp.curry)(function (removedItems, state) {
  return {
    items: state.items,
    selected: (0, _fp.without)(removedItems, state.selected),
    changed: {
      selected: [],
      deselected: (0, _fp.intersection)(removedItems, state.selected)
    },
    anchor: null
  };
});

var removeAll = (0, _fp.curry)(function (state) {
  return {
    items: state.items,
    selected: [],
    changed: {
      selected: [],
      deselected: state.selected
    },
    anchor: null
  };
});

var rangeTo = (0, _fp.curry)(function (toItem, state) {
  if (!(0, _fp.includes)(toItem, state.items)) {
    return state;
  }

  var anchor = _getAnchor(state, state.items);
  var connected = _connectedWith(anchor, state.selected, state.items);
  var range = _between(anchor, toItem, state.items);

  var selected = (0, _fp.flow)((0, _fp.without)(connected), (0, _fp.union)(range))(state.selected);

  return {
    items: state.items,
    selected: selected,
    changed: {
      selected: (0, _fp.without)(state.selected, selected),
      deselected: (0, _fp.without)(selected, state.selected)
    },
    anchor: state.anchor
  };
});

function _between(start, end, array) {
  if (start === end) {
    return [start];
  }

  var startIndex = array.indexOf(start);
  var endIndex = array.indexOf(end);

  if (startIndex > endIndex) {
    var temp = startIndex;
    startIndex = endIndex;
    endIndex = temp;
  }

  return array.slice(startIndex, endIndex + 1);
}

function _connectedWith(targetItem, selected, array) {
  var isSelected = function isSelected(item) {
    return selected.indexOf(item) !== -1;
  };
  var result = [];
  var targetIndex = array.indexOf(targetItem);

  for (var i = targetIndex; i >= 0; i--) {
    if (!isSelected(array[i])) {
      break;
    }
    result.push(array[i]);
  }

  for (var _i = targetIndex; _i < array.length; _i++) {
    if (!isSelected(array[_i])) {
      break;
    }
    result.push(array[_i]);
  }

  return result;
}

exports.init = init;
exports.setItems = setItems;
exports.setSelection = setSelection;
exports.replace = replace;
exports.toggle = toggle;
exports.remove = remove;
exports.removeAll = removeAll;
exports.rangeTo = rangeTo;
exports.getItems = getItems;
exports.getSelection = getSelection;
exports.getChangedSelection = getChangedSelection;
exports.getChangedDeselection = getChangedDeselection;